__all__ = ['mf_partio43delight_RI']

import maya.cmds as mc
import maya.mel as mel
import string
import re

def mf_partio43delight_addExtraAttrs(shape):
	if( not mc.objExists(shape + ".particle_type") ):
		mc.addAttr(shape, ln="particle_type", at="enum", enumName="particle:sphere:disk:patch:blobby")

	if( not mc.objExists(shape + ".radius") ):
		mc.addAttr(shape, ln="radius", at="float")
		mc.setAttr(shape + ".radius", k=1)
		mc.setAttr(shape + ".radius", 1)
	
	if( not mc.objExists(shape + ".radius_scale") ):
		mc.addAttr(shape, ln="radius_scale", at="float")
		mc.setAttr(shape + ".radius_scale", k=1)
		mc.setAttr(shape + ".radius_scale", 1)

	if( not mc.objExists(shape + ".radius_space") ):
		mc.addAttr(shape, ln="radius_space", at="enum", enumName="raster:object:world")

	if( not mc.objExists(shape + ".velocity_scale") ):
		mc.addAttr(shape, ln="velocity_scale", at="float")
		mc.setAttr(shape + ".velocity_scale", k=1)
		mc.setAttr(shape + ".velocity_scale", 0.04) # 25fps

	if( not mc.objExists(shape + ".replicate") ):
		mc.addAttr(shape, ln="replicate", at="long")
		mc.setAttr(shape + ".replicate", k=1)

	if( not mc.objExists(shape + ".jitter") ):
		mc.addAttr(shape, ln="jitter", at="float")
		mc.setAttr(shape + ".jitter", k=1)

	if( not mc.objExists(shape + ".jitter_adaptive") ):
		mc.addAttr(shape, ln="jitter_adaptive", at="bool")
		mc.setAttr(shape + ".jitter_adaptive", k=1)
		
	if( not mc.objExists(shape + ".max_jitter") ):
		mc.addAttr(shape, ln="max_jitter", at="float")
		mc.setAttr(shape + ".max_jitter", k=1)
		mc.setAttr(shape + ".max_jitter", -1)

	if( not mc.objExists(shape + ".RenderPartitions") ):
		mc.addAttr(shape, ln="RenderPartitions", at="bool")
		mc.setAttr(shape + ".RenderPartitions", k=1)

	if( not mc.objExists(shape + ".MaximumPartitions") ):
		mc.addAttr(shape, ln="MaximumPartitions", at="long")
		mc.setAttr(shape + ".MaximumPartitions", k=1)

	if( not mc.objExists(shape + ".MergePartitions") ):
		mc.addAttr(shape, ln="MergePartitions", at="bool")
		mc.setAttr(shape + ".MergePartitions", k=1)

	if( not mc.objExists(shape + ".renderCachePath") ):
		mc.addAttr(shape, ln="renderCachePath", dt="string")
		
	if( not mc.objExists(shape + ".bboxExpand") ):
		mc.addAttr(shape, ln="bboxExpand", at="float")

	if( not mc.objExists(shape + ".time") ):
		mc.addAttr(shape, ln="time", at="float")
		mc.setAttr(shape + ".time", k=1)


def PartitionNameToTemplate(i_path):
	reg = re.compile('.*part(\d+)of\d+', re.IGNORECASE)
	match = reg.match(i_path)
	result = i_path
	if( match ):
		#partition = match.group(1)
		result = i_path[:match.start(1)] + "$PART" + i_path[match.end(1):]
	return result

def mf_partio43delight_RI(visNode):
	print "mf_partio43delight_RI: " + visNode
	mf_partio43delight_addExtraAttrs(visNode)

	cacheFile = mc.getAttr(visNode + ".renderCachePath")
	frame = string.zfill(int(mc.currentTime(q=1)),4)
	cacheFile = cacheFile.replace("#", frame)
	cacheFile = cacheFile.replace("<frame>", frame)

	#Procedural "DynamicLoad" ["E:/dev/partio43delight/x64/Release/partio43delight" "--file C:/temp/partio/bareShape.0023.geo --replicate 20 --radius 4 --jitter 1.0"] [-1000 1000 -1000 1000 -1000 1000]
	RES = 'Procedural ' + '"DynamicLoad" '
	RES += '["partio43delight_procedural" '

	PARAM_STR = ""
	PARAM_STR += " --type " + mc.getAttr(visNode + ".particle_type", asString=1)
	PARAM_STR += " --radius " + str(mc.getAttr(visNode + ".radius"))
	PARAM_STR += " --radius_scale " + str(mc.getAttr(visNode + ".radius_scale"))
	PARAM_STR += " --radius_space " + mc.getAttr(visNode + ".radius_space", asString=1)	
	PARAM_STR += " --velocity_scale " + str(mc.getAttr(visNode + ".velocity_scale"))

	if( mc.objExists(visNode + ".RenderPartitions") and mc.objExists(visNode + ".MaximumPartitions") and mc.getAttr(visNode + ".RenderPartitions") ):
		PARAM_STR += " --partitions " + str(mc.getAttr(visNode + ".MaximumPartitions"))
		PARAM_STR += " --file " + PartitionNameToTemplate(cacheFile)
		if( mc.getAttr(visNode + ".MergePartitions") ):
			PARAM_STR += " --merge_partitions"
	elif( mc.getAttr(visNode + ".replicate") ):
		PARAM_STR += " --replicate " + str(mc.getAttr(visNode + ".replicate"))
		PARAM_STR += " --jitter " + str(mc.getAttr(visNode + ".jitter"))
		if( mc.getAttr(visNode + ".jitter_adaptive") ):
			PARAM_STR += " --jitter_adaptive"
		PARAM_STR += " --max_jitter " + str(mc.getAttr(visNode + ".max_jitter"))
		PARAM_STR += " --file " + cacheFile
		if( mc.getAttr(visNode + ".MergePartitions") ):
			PARAM_STR += " --merge_partitions"
	else:
		PARAM_STR += " --file " + cacheFile

	RES += '"' + PARAM_STR + '"]'

	# bbox
	bboxExpand = mc.getAttr(visNode + ".bboxExpand")
	bbox_min = list(mc.getAttr(visNode + ".boundingBoxMin")[0])
	bbox_max = list(mc.getAttr(visNode + ".boundingBoxMax")[0])
	bbox_min[0] -= bboxExpand
	bbox_min[1] -= bboxExpand
	bbox_min[2] -= bboxExpand
	bbox_max[0] += bboxExpand
	bbox_max[1] += bboxExpand
	bbox_max[2] += bboxExpand
	map(str,bbox_min)
	map(str,bbox_max)	
	bbox = "%s %s %s %s %s %s" % ( bbox_min[0], bbox_max[0], bbox_min[1], bbox_max[1], bbox_min[2], bbox_max[2] )
	#bbox += str(mc.getAttr(visNode + ".boundingBoxMinX")) + " " + str(mc.getAttr(visNode + ".boundingBoxMaxX")) + " "
	#bbox += str(mc.getAttr(visNode + ".boundingBoxMinY")) + " " + str(mc.getAttr(visNode + ".boundingBoxMaxY")) + " "
	#bbox += str(mc.getAttr(visNode + ".boundingBoxMinZ")) + " " + str(mc.getAttr(visNode + ".boundingBoxMaxZ")) + " "

	RES += '[' + bbox + ']'
	RES += "\n"

	return RES